/****************************************************************************
 ** This file is part of the MAIA (Malagasy Artificial Intelligence Art) Open Project.
 **
 ** Copyright (C) 2012, 2013 ANDRIAMILAMINA Monge.
 ** All rights reserved.
 **
 ** License: GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007.
 ** Author:  ANDRIAMILAMINA Monge.
 ** Contact: mongemilamina@gmail.com / +261 34 45 062 85.
 ** Year:    2013.
 ** Status:  MAIA Senior Engineer, Madagasikara, Antananarivo.
 ** Domain:  Automatic.
 **
 ** Commercial Usage
 ** Please contact MAIA.
 **
 ** GNU General Public License Usage
 ** This file may be used under the terms of the GNU
 ** General Public License version 3.0 as published by the Free Software
 ** Foundation and appearing in the file LICENSE.GPL included in the
 ** packaging of this file.  Please review the following information to
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** If you have questions regarding the use of this file, please contact
 ** the Author.
 **
 ****************************************************************************/

#include "dirview.h"
#include "ui_dirview.h"
#include <QString>
#include <QStringList>
#include <QFileDialog>
#include <QDir>
#include <QFile>
#include <QIODevice>
#include <QIcon>
#include <QMessageBox>
#include <QEvent>
#include <QCloseEvent>
#include <QLabel>


DirView::DirView(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::DirView)
{
    ui->setupUi(this);

    myUrlLabel = new QLabel("<b><a href='https://www.andriamilamina.skyrock.com'>" + tr("MAIA Website") + "</a></b>", this);
    myUrlLabel->setToolTip(tr("Go to the MAIA website"));
    myUrlLabel->setOpenExternalLinks(true);

    //system tray handlling
    //#############################################################
    myTray = new QSystemTrayIcon(QIcon(":/qrc_icons/icons/MAIA_icon_Synch_Tray.png"), this);
    if(!myTray->isSystemTrayAvailable())
    {
        QMessageBox::critical(this, tr("Error!"), tr("Failed to get access to the system tray!"));
    }
    else
    {
        myTray->show();
        myTray->setToolTip("MAIA Sync 1.0");
    }
    //#############################################################

    QStringList boxList;
    boxList << "*.mp3" << "*.html" << "*.txt" << "*.pdf" << "*.tar.gz" << "*.djvu" << "*.odt" << "*.ogg" << "*.wav" << "*.jpg" << "*.png" << "*";
    myComboFilter = new QComboBox();
    myComboFilter->setEditable(true);
    myComboFilter->addItems(boxList);

    myTrayMenu = new QMenu(this);

    myActionNew = new QAction(QIcon(":/qrc_icons/icons/MAIA_icon_New.png"), tr("&New"), this);
    myActionNew->setIconText(tr("&New"));
    myActionNew->setShortcut(Qt::CTRL+Qt::Key_N);

    myActionSynch = new QAction(QIcon(":/qrc_icons/icons/MAIA_icon_Synch.png"), tr("&Sync"), this);
    myActionSynch->setIconText(tr("&Sync"));
    myActionSynch->setShortcut(Qt::Key_F5);

    myActionAboutAndHelp = new QAction(QIcon(":/qrc_icons/icons/MAIA_icon_About.png"), tr("&About"), this);
    myActionAboutAndHelp->setIconText(tr("&About"));
    myActionAboutAndHelp->setShortcut(Qt::Key_F1);

    myActionQuit = new QAction(QIcon(":/qrc_icons/icons/MAIA_icon_Quit.png"), tr("&Quit"), this);
    myActionQuit->setIconText(tr("&Quit"));
    myActionQuit->setShortcut(Qt::CTRL+Qt::Key_Q);

    myTrayMenu->addAction(myActionNew);
    myTrayMenu->addAction(myActionSynch);
    myTrayMenu->addSeparator();
    myTrayMenu->addAction(myActionAboutAndHelp);
    myTrayMenu->addSeparator();
    myTrayMenu->addAction(myActionQuit);

    ui->toolBar->addAction(myActionNew);
    ui->toolBar->addAction(myActionSynch);
    ui->toolBar->addWidget(myComboFilter);
    ui->toolBar->addSeparator();
    ui->toolBar->addAction(myActionAboutAndHelp);
    ui->toolBar->addSeparator();
    ui->toolBar->addAction(myActionQuit);
    ui->toolBar->addSeparator();
    ui->toolBar->addWidget(myUrlLabel);

    myTray->setContextMenu(myTrayMenu);

    ui->label_counter->setText("--- / ---");

    connect(ui->b_browseHDD, SIGNAL(clicked()), this, SLOT(browseHDD()));
    connect(ui->b_browseUSB, SIGNAL(clicked()), this, SLOT(browseUSB()));

    connect(myComboFilter, SIGNAL(currentIndexChanged(QString)), this, SLOT(handleFilter()));
    connect(myComboFilter, SIGNAL(editTextChanged(QString)), this, SLOT(handleFilter()));
    connect(myTray, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(trayIconActivated(QSystemTrayIcon::ActivationReason)));

    connect(myActionNew, SIGNAL(triggered()), this, SLOT(newSynch()));
    connect(myActionAboutAndHelp, SIGNAL(triggered()), this, SLOT(aboutAndHelp()));
    connect(myActionQuit, SIGNAL(triggered()), this, SLOT(close()));
    connect(myActionSynch, SIGNAL(triggered()), this, SLOT(synch()));

    this->newSynch();

}
//#############################################################
void DirView::newSynch()
{
    //reset UIs
    myComboFilter->setCurrentIndex(0);
    //ui->progressBar->reset();
    ui->progressBar->setValue(0);
    ui->lab_curr_file->clear();//clear label of current file for not disturbing the progress bar
    //filter listing at startup
    QStringList filterFromUIFilter;//create a string list to contain the filter
    filterFromUIFilter << myComboFilter->currentText();//get the ui combobox current text as the filter

    ui->label_pathHDD->setText(QDir::homePath());// re-mettre le chemin du model sur un label
    ui->label_pathHDD->setToolTip(ui->label_pathHDD->text());

    //handlling the reset of the label counter
    QDir myHDDDir;//create a dirctory object
    myHDDDir.setNameFilters(filterFromUIFilter);
    myHDDDir.setPath(QDir::homePath());//set the path of the dir to the HDDbrowsed path
    int HDDItems = myHDDDir.count();//count items in the browsed dir
    ui->label_counter->setText("0 / " + QString::number(HDDItems, 10));//set it as a text onto the counter label

    myTray->showMessage("MAIA Sync 1.0", tr("Ready for a new synchronization!\nFile filter, set to ") + myComboFilter->currentText(), QSystemTrayIcon::Information, 1000);

//--------------------------------------------------------------------
    ui->label_pathUSB->setText(QDir::homePath());// re-mettre le chemin du model sur un label
    ui->label_pathUSB->setToolTip(ui->label_pathUSB->text());    
}
//#############################################################
void DirView::handleFilter()
{
    QStringList filterFromUIFilter;//create a string list to contain the filter
    filterFromUIFilter << myComboFilter->currentText();//get the ui combobox current text as the filter

    QDir myHDDDir;//create a dirctory object
    myHDDDir.setNameFilters(filterFromUIFilter);//filter listing at each filter changement
    myHDDDir.setPath(ui->label_pathHDD->text());//set the path of the dir to the HDDbrowsed path
    int HDDItems = myHDDDir.count();//count items in the browsed dir

    ui->label_pathHDD->setToolTip(ui->label_pathHDD->text());

    ui->label_pathHDD->setText(ui->label_pathHDD->text());// mettre le chemin du model sur un label

    ui->label_counter->setText("0 / " + QString::number(HDDItems, 10));//set it as a text onto the counter label
}
//#############################################################
void DirView::browseHDD()
{

    QFileDialog dialog; // crer un file dialog
    QString pathHDD_Browsed; // crer un string pour le chemin du file dialog
    pathHDD_Browsed = dialog.getExistingDirectory(this, tr("Choose the source directory..."), QDir::homePath());// affecter le chemin eu par getExistingDirectory dans le string

    if(pathHDD_Browsed.isEmpty())
        return;

    ui->label_pathHDD->setToolTip(pathHDD_Browsed);
    ui->label_pathHDD->setText(pathHDD_Browsed);// mettre le chemin du model sur un label


    //handlling counter label after browsing, ONLY For Browse HDD
    QDir myHDDDir;//create a dirctory object
    myHDDDir.setPath(pathHDD_Browsed);//set the path of the dir to the HDDbrowsed path
    int HDDItems = myHDDDir.count();//count items in the browsed dir
    ui->label_counter->setText("0 / " + QString::number(HDDItems, 10));//set it as a text onto the counter label

    this->handleFilter();//handle filtering after browsing
}
//#############################################################
void DirView::browseUSB()
{
    QFileDialog dialog; // crer un file dialog
    QString pathUSB_Browsed; // crer un string pour le chemin du file dialog
    pathUSB_Browsed = dialog.getExistingDirectory(this, tr("Choose the destination directory..."), QDir::homePath());// affecter le chemin eu par getExistingDirectory dans le string

    if(pathUSB_Browsed.isEmpty())
        return;

    ui->label_pathUSB->setToolTip(pathUSB_Browsed);
    ui->label_pathUSB->setText(pathUSB_Browsed);// mettre le chemin du model sur un label
}
//#############################################################
void DirView::synch()
{

    QStringList hddFilter, hddList;//create filter and list
    hddFilter << myComboFilter->currentText();//get the filter criteria
    QDir hddDir(ui->label_pathHDD->text());//create a directory with it's path

    hddList = hddDir.entryList(hddFilter);//get a filtered directory entries in the list previously created
    int hddItems = hddList.size();//count the total item in the directory and put it in an integer

    //copying
    for (int i = 0; i < hddItems; i++)
    {
        QString srcPath = ui->label_pathHDD->text() + "/" + hddList.at(i);//the source path
        QString destPath = ui->label_pathUSB->text() + "/" + hddList.at(i);//the destination path

        //hddList.at(i) returns the filename
        //ui->textEdit->append(hddList.at(i));

        //srcPath returns the absolute file path
        //ui->textEdit->append(destPath);
        QFile myFile(srcPath);//the source file to copy

        if (!myFile.open(QIODevice::ReadOnly))//test if can write
        {
            QMessageBox::critical(this, tr("Warning..."), tr("Copy operation failed!"));
            return;
        }
        myFile.copy(destPath);//copy the source file to the given path
        ui->progressBar->setValue(((i+1) * 100) / hddItems);//update progress bar value
        ui->label_counter->setText(QString::number(i+1, 10) + " / " + QString::number(hddItems, 10));//set it as a text onto the counter label

//##########################################
        ui->lab_curr_file->setText(tr("Copying... ") + hddList.at(i));//update the label of current file
//##########################################

        myFile.close();//close the file after each copy
    }
    ui->lab_curr_file->setText(tr("End of operation!"));//clear label of current file for not disturbing the progress bar
    myTray->showMessage("MAIA Sync 1.0", tr("End of operation!"), QSystemTrayIcon::Information, 3000);

}
//#############################################################
void DirView::aboutAndHelp()
{
    myAboutText = new QTextEdit(this);//create a textEdit object for about dialog
    myAboutText->setReadOnly(true);
    myAboutText->setText("\tMAIA Sync 1.0 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. \n\n\tMAIA Sync is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details. \n\n\tYou should have received a copy of the GNU General Public License along with MAIA Sync; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA");

    myHelpText = new QTextEdit(this);//create a textEdit object for help dialog
    myHelpText->setReadOnly(true);
    myHelpText->setText(tr("MAIA Sync 1.0 helps you to synchronize two folders.\nYou can use it to synchronize, for instance, a folder on your hard drive and another one on an external drive. \n\n1. You have to browse the source folder. \n2. You have to browse the destination folder. \n3. You can lauch the synchronization by:\n\t3-1. The \"Sync\" button or \n\t3-2. Choose \"Sync\" in the tray icon menu or \n\t3-3. Hit F5. \n\n4. To run another task, click on \"New\" button or on \"New\" in the tray icon menu.\n\nAuthor: ANDRIAMILAMINA Monge\nE-mail: mongemilamina@gmail.com"));

    myTabWidget = new QTabWidget();//no parent, so this is gonna have its own window
    myTabWidget->setWindowFlags(Qt::WindowStaysOnTopHint);//make the tabwidget above other application windows
    myTabWidget->setAttribute(Qt::WA_ShowModal, true);//make the TabWidget Modal
    myTabWidget->insertTab(1, myAboutText, QIcon(":/qrc_icons/icons/MAIA_icon_About.png"), tr("&About"));//adding the appropriate label to the tabBar, with its title
    myTabWidget->insertTab(1, myHelpText, QIcon(":/qrc_icons/icons/MAIA_icon_Help.png"), tr("&Help"));//adding the appropriate label to the tabBar, with its title

    myTabWidget->setWindowTitle(tr("MAIA Sync 1.0, About and Help"));
    myTabWidget->setWindowIcon(QIcon(":/qrc_icons/icons/MAIA_icon_Synch_Tray.png"));
    myTabWidget->setMinimumSize(500, 300);
    myTabWidget->show();
}
//#############################################################
void DirView::closeEvent(QCloseEvent *ev)//wait for a QCloseEvent *ev signal sent by Window Manager close button click
{
    if(ev->type() == QEvent::Close)
    {
        myTray->showMessage("MAIA Sync 1.0", tr("About to quit"), QSystemTrayIcon::Warning, 1000);


        QMessageBox::StandardButton reponse;
            reponse = QMessageBox::warning(this, tr("MAIA Sync 1.0"), tr("Quit the application?"), QMessageBox::No | QMessageBox::Yes);
            if (reponse == QMessageBox::No)
            {
                ev->ignore();
                if(this->isHidden())
                this->show();
            }
            else if (reponse == QMessageBox::Yes)
                this->close();
    }
}
//#############################################################
void DirView::trayIconActivated(QSystemTrayIcon::ActivationReason reas)//get the activation reason of the trayicon (clicked or right clicked ...)
{
    if(this->isHidden())//test if application is hidden or not
    {
        switch (reas)
        {
            case QSystemTrayIcon::Trigger:
            this->show();//if it is hidden, show it
            myTray->setToolTip("MAIA Sync 1.0");
            break;
            default:
            break;
        }
    }
    else if(!this->isHidden())
    {
        switch (reas)
        {
            case QSystemTrayIcon::Trigger:
            this->hide();//if it is shown, hide it
            myTray->setToolTip(tr("MAIA Sync 1.0 is minimized to system tray!"));
            myTray->showMessage("MAIA Sync 1.0", tr("MAIA Sync 1.0 is minimized to system tray!"), QSystemTrayIcon::Information, 3000);
            break;
            default:
            break;
        }
    }
}
//#############################################################
DirView::~DirView()
{
    delete ui;
}
//#############################################################
void DirView::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}
//#############################################################
